## SR860/SR865A Data Capture and Streaming with Python

There are two scripts in this repo: stream.py and cap860.py They both collect data from an SRS lock-in amplifier.  

* stream does it in real-time using UDP over an ethernet connection.  
* cap860 uses the lock-in capture buffer before downloading the data to the host computer.  

cap860.py is detailed after stream.py

## What's a lock-in amplifier?
A lock-in is a scientific instrument for measuring the phase and amplitude of a signal at a specific frequency.
It uses a Phase Synchronous Detector (PSD) to extract the in-phase and quadrature components of the signal.
[This application note](http://www.thinksrs.com/downloads/PDFs/ApplicationNotes/AboutLIAs.pdf) explains it in much more detail.

## Which script should I use?
* stream.py uses UDP which may drop packets (data) depending on network traffic so if missing data is a problem, you might want to use cap860.py  
* cap860.py transfers using TCP/IP so packets are re-transmitted as needed and no data is lost. 
However, the cap860.py transfer happens after the data collection is complete, so you must wait until the capture ends.

# stream.py
## What is this?
This python script streams data from a Stanford Research Systems an [SR860](https://www.thinksrs.com/products/sr860.html), SR865, or [SR865A](http://www.thinksrs.com/products/SR865A.htm) lock-in amplifier.

## How do I run this script?
You need a few things:  

* A computer with python  
   * I've tested this with python 3.7.3 and 3.11.0  
   * you'll need to install vxi11 using pip, brew, or whatever installation tool you use.  
   * The script parses command line arguments with docopt, so you'll need to install that, too.  

In either your global Python environment or an active virtual environment, you can use
```powershell
PS ..\python> pip install -r requirements.txt
```

* an [SR860](https://www.thinksrs.com/products/sr860.html), SR865, or [SR865A](http://www.thinksrs.com/products/SR865A.htm)
* a network connection between the two.

## Is that all?
You also need to:

* open your computer's firewall for incoming UDP on the streaming port. This script defaults to 1865 but any port that doesn't conflict with some other service will work.  
* set the IP address of the SR865 to be visible to your computer and not conflict with other devices on your network (from the instrument front panel).   

Both of these things may require help from your local IT person.

* use telnet to verify that your computer can connect to the SR865.
```powershell
PS> telnet
```

```telnet
Welcome to Microsoft Telnet Client

Escape Character is 'CTRL+]'

Microsoft Telnet>open 172.25.129.22
```

```telnet
SR865 Lockin Amplifier: Connection 1
```

- Type `*idn?` in telnet and you should see the identification string.


```telnet
*idn?

Stanford_Research_Systems,SR860,002819,v1.51
```

* Exit telnet: Hold the `CTRL` key and press `]` to get back to the telnet prompt, then type `quit`.

Once you've done these things, it's a good idea to edit the default settings in stream.py. 
The settings start around line 66. 
(That is, modify the `default` values found in the `USE_STR` string variable starting at line 60, such as the IP address at line 67. Change the `XXX.XX.XXX.XX` characters to match your instrument's IP address).

* Change the default IP to match the setting in the SR865 Setup/Ethernet menu. 
* Make sure the UDP Port setting matches the opened port in your firewall.

## Example usage
```powershell
(venv) PS ..\python> py stream.py --duration=10 --file=test.dat

opening incoming UDP Socket at 1865 ... done
opening VXI-11 at 172.25.129.22 ... done
Max rate is 2.441 kS/S.
Decimating by 2^0 down to 2.441 kS/S
  test.dat written
 cleaning up... connections closed


pass: No packets dropped out of 48. 12288 samples captured.
Time elapsed: 5.083 seconds
```
At this point, the file test.dat should have the desired data in it.

Of course, you can specify values for any of the options on the command line when the call to `stream.py` is made:

```powershell
(venv) PS ..\python> py stream.py -a 172.25.129.22 -p 1865 -d 10 -f test2.dat

opening incoming UDP Socket at 1865 ... done
opening VXI-11 at 172.25.129.22 ... done
Max rate is 2.441 kS/S.
Decimating by 2^0 down to 2.441 kS/S
  test2.dat written
 cleaning up... connections closed


pass: No packets dropped out of 96. 24576 samples captured.
Time elapsed: 10.121 seconds
```


# cap860.py
## What is this?
This python script configures the instrument to capture data internally and, when complete, download it to the host computer.

## How do I run this script?
You need all the things mentioned for stream.py, above.

## Is that all?
Like stream.py, you *do* need to set the instrument IP address but you *do not* need to open a port in your firewall. Follow the instructions for stream but ignore the stuff about the port.

## Example usage
```powershell
(venv) PS ..\python> py cap860.py -a 172.25.129.22 -c 500 -f captest.dat
opening VXI-11 at 172.25.129.22 ... done
 capture took 0.234 seconds. Retrieving data...
  captest.dat written
 cleaning up... connections closed
```
